@@ -282,7 +282,7 @@ class UpdatesAvailable(InternalDialog):
         self.pkg_cell_area = CellAreaPackage(False)
         pkg_column = Gtk.TreeViewColumn.new_with_area(self.pkg_cell_area)
         self.pkg_cell_area.column = pkg_column
-        pkg_column.set_title(_("Install"))
+        pkg_column.set_title(_("Install or remove"))
         pkg_column.set_property("spacing", 4)
         pkg_column.set_expand(True)
         self.treeview_update.append_column(pkg_column)
@@ -531,12 +531,12 @@ class UpdatesAvailable(InternalDialog):
         # set descr
         data = model.get_value(iter, LIST_UPDATE_DATA)
         item = data.item
-        if (item is None and data.group is not None and
-                data.group.core_item is not None):
+        if (item is None and data.group is not None
+                and data.group.core_item is not None):
             item = data.group.core_item
-        if (item is None or item.pkg is None or
-                item.pkg.candidate is None or
-                item.pkg.candidate.description is None):
+        if (item is None or item.pkg is None
+                or item.pkg.candidate is None
+                or item.pkg.candidate.description is None):
             changes_buffer = self.textview_changes.get_buffer()
             changes_buffer.set_text("")
             desc_buffer = self.textview_descr.get_buffer()
@@ -632,7 +632,7 @@ class UpdatesAvailable(InternalDialog):
                 Gtk.MenuItem.new_with_mnemonic(_("_Deselect All"))
             item_select_none.connect("activate", self.select_none_upgrades)
             menu.append(item_select_none)
-            num_updates = self.cache.install_count
+            num_updates = self.cache.install_count + self.cache.del_count
             if num_updates == 0:
                 item_select_none.set_property("sensitive", False)
             item_select_all = Gtk.MenuItem.new_with_mnemonic(_("Select _All"))
@@ -724,9 +724,10 @@ class UpdatesAvailable(InternalDialog):
         self.hbox_restart.set_visible(requires_restart)
 
     def _refresh_updates_count(self):
-        self.button_install.set_sensitive(self.cache.install_count)
+        self.button_install.set_sensitive(self.cache.install_count
+                                          + self.cache.del_count)
         try:
-            inst_count = self.cache.install_count
+            inst_count = self.cache.install_count + self.cache.del_count
             self.dl_size = self.cache.required_download
             download_str = ""
             if self.dl_size != 0:
@@ -874,7 +875,7 @@ class UpdatesAvailable(InternalDialog):
                                            remedy_usr)
             self.window_main.start_error(False, err_sum, err_long)
             return
-        except SystemError as e:
+        except SystemError:
             logging.exception("free space check failed")
         self.window_main.start_install()
 
@@ -888,8 +889,8 @@ class UpdatesAvailable(InternalDialog):
             self.vbox_alerts.show()
             self.connected = False
         # in doubt (STATE_UNKNOWN), assume connected
-        elif (state in NetworkManagerHelper.NM_STATE_CONNECTED_LIST or
-              state == NetworkManagerHelper.NM_STATE_UNKNOWN):
+        elif (state in NetworkManagerHelper.NM_STATE_CONNECTED_LIST
+              or state == NetworkManagerHelper.NM_STATE_UNKNOWN):
             self.updates_changed()
             self.hbox_offline.hide()
             self.connected = True
@@ -955,7 +956,10 @@ class UpdatesAvailable(InternalDialog):
                 if keep_packages:
                     item.pkg.mark_keep()
                 elif item.pkg.name not in self.list.held_back:
-                    item.pkg.mark_install()
+                    if not item.to_remove:
+                        item.pkg.mark_install()
+                    else:
+                        item.pkg.mark_delete()
             except SystemError:
                 pass
 
@@ -1051,11 +1055,27 @@ class UpdatesAvailable(InternalDialog):
             self._add_groups(self.list.security_groups)
         if self.list.security_groups and self.list.update_groups:
             self._add_header(_("Other updates"), self.list.update_groups)
+        elif self.list.update_groups and (self.list.kernel_autoremove_groups
+                                          or self.list.duplicate_groups):
+            self._add_header(_("Updates"), self.list.update_groups)
         if self.list.update_groups:
             self._add_groups(self.list.update_groups)
+        if self.list.kernel_autoremove_groups:
+            self._add_header(
+                _("Unused kernel updates to be removed"),
+                self.list.kernel_autoremove_groups)
+            self._add_groups(self.list.kernel_autoremove_groups)
+        if self.list.duplicate_groups:
+            self._add_header(
+                _("Duplicate packages to be removed"),
+                self.list.duplicate_groups)
+            self._add_groups(self.list.duplicate_groups)
 
         self.treeview_update.set_model(self.store)
-        self.pkg_cell_area.indent_toplevel = bool(self.list.security_groups)
+        self.pkg_cell_area.indent_toplevel = (
+            bool(self.list.security_groups)
+            or bool(self.list.kernel_autoremove_groups)
+            or bool(self.list.duplicate_groups))
         self.update_close_button()
         self.update_count()
         self.setBusy(False)
